@@ -449,10 +449,10 @@ as_util_convert_appdata (GFile *file_input,
 			      NULL, error))
 		return FALSE;
 
-	/* TRANSLATORS: any manual changes required? */
 	if (action_required) {
-		g_print (_("Please review the file and fix any '%s' items"), "FIXME");
-		g_print ("\n");
+		/* TRANSLATORS: any manual changes required?
+		 * also note: FIXME is a hardcoded string */
+		g_print ("%s\n", _("Please review the file and fix any 'FIXME' items"));
 	}
 
 	return TRUE;
@@ -483,7 +483,7 @@ as_util_convert_appstream (GFile *file_input,
 				NULL, error))
 		return FALSE;
 	/* TRANSLATORS: information message */
-	g_print (_("New API version: %.2f\n"), as_store_get_api_version (store));
+	g_print ("%s: %.2f\n", _("New API version"), as_store_get_api_version (store));
 	return TRUE;
 }
 
@@ -625,16 +625,16 @@ as_util_appdata_from_desktop (AsUtilPrivate *priv, gchar **values, GError **erro
 	as_app_set_project_group (app, "XXX: Values valid are none, GNOME, KDE or XFCE", -1);
 
 	/* fix the ID */
-	id_new = g_strdup (as_app_get_id_full (app));
+	id_new = g_strdup (as_app_get_id (app));
 	instr = g_strstr_len (id_new, -1, ".desktop.in");
 	if (instr != NULL) {
 		instr[8] = '\0';
-		as_app_set_id_full (app, id_new, -1);
+		as_app_set_id (app, id_new, -1);
 	}
 
 	/* set things that don't belong in the AppData file */
 	as_app_set_icon (app, NULL, -1);
-	g_ptr_array_set_size (as_app_get_keywords (app), 0);
+	g_ptr_array_set_size (as_app_get_keywords (app, NULL), 0);
 	g_ptr_array_set_size (as_app_get_categories (app), 0);
 	g_ptr_array_set_size (as_app_get_mimetypes (app), 0);
 
@@ -942,8 +942,13 @@ as_util_install_filename (const gchar *filename,
 
 	switch (as_app_guess_source_kind (filename)) {
 	case AS_APP_SOURCE_KIND_APPSTREAM:
-		ret = as_util_install_xml (filename, origin,
-					   "/usr/share/app-info/xmls", error);
+		if (g_strstr_len (filename, -1, ".yml.gz") != NULL) {
+			ret = as_util_install_xml (filename, origin,
+						   "/usr/share/app-info/yaml", error);
+		} else {
+			ret = as_util_install_xml (filename, origin,
+						   "/usr/share/app-info/xmls", error);
+		}
 		break;
 	case AS_APP_SOURCE_KIND_APPDATA:
 	case AS_APP_SOURCE_KIND_METAINFO:
@@ -1179,7 +1184,7 @@ as_util_status_html_write_app (AsApp *app, GString *html, AsUtilDistro distro)
 		g_string_append (classes, "screenshots ");
 	if (as_app_get_description(app, NULL) != NULL)
 		g_string_append (classes, "description ");
-	if (as_app_get_keywords(app)->len > 0)
+	if (as_app_get_keywords(app, NULL) != NULL)
 		g_string_append (classes, "keywords ");
 	if (g_strcmp0 (as_app_get_project_group (app), "GNOME") == 0)
 		g_string_append (classes, "gnome ");
@@ -1286,12 +1291,14 @@ as_util_status_html_write_app (AsApp *app, GString *html, AsUtilDistro distro)
 	g_free (tmp);
 
 	/* keywords */
-	tmp = as_util_status_html_join (as_app_get_keywords (app));
-	if (tmp != NULL) {
-		g_string_append_printf (html, "<tr><td class=\"alt\">%s</td><td>%s</td></tr>\n",
-					"Keywords", tmp);
+	if (as_app_get_keywords (app, NULL) != NULL) {
+		tmp = as_util_status_html_join (as_app_get_keywords (app, NULL));
+		if (tmp != NULL) {
+			g_string_append_printf (html, "<tr><td class=\"alt\">%s</td><td>%s</td></tr>\n",
+						"Keywords", tmp);
+		}
+		g_free (tmp);
 	}
-	g_free (tmp);
 
 	/* homepage */
 	pkgname = as_app_get_url_item (app, AS_URL_KIND_HOMEPAGE);
@@ -1333,6 +1340,16 @@ as_util_status_html_write_app (AsApp *app, GString *html, AsUtilDistro distro)
 		g_free (tmp);
 	}
 
+	/* vetos */
+	if (as_app_get_id_kind (app) == AS_ID_KIND_DESKTOP) {
+		tmp = as_util_status_html_join (as_app_get_vetos (app));
+		if (tmp != NULL) {
+			g_string_append_printf (html, "<tr><td class=\"alt\">%s</td><td>%s</td></tr>\n",
+						"Vetos", tmp);
+		}
+		g_free (tmp);
+	}
+
 	g_string_append (html, "</table>\n");
 	g_string_append (html, "<br/>\n");
 	g_string_append (html, "</div>\n");
@@ -1392,7 +1409,7 @@ as_util_status_html_write_exec_summary (GPtrArray *apps,
 		app = g_ptr_array_index (apps, i);
 		if (as_app_get_id_kind (app) != AS_ID_KIND_DESKTOP)
 			continue;
-		if (as_app_get_keywords(app)->len > 0)
+		if (as_app_get_keywords(app, NULL) != NULL)
 			cnt++;
 	}
 	perc = 100.f * (gdouble) cnt / (gdouble) total;
@@ -1704,6 +1721,7 @@ static gboolean
 as_util_status_csv (AsUtilPrivate *priv, gchar **values, GError **error)
 {
 	AsApp *app;
+	AsIdKind id_kind = AS_ID_KIND_DESKTOP;
 	GPtrArray *apps = NULL;
 	guint i;
 	_cleanup_object_unref_ AsStore *store = NULL;
@@ -1711,7 +1729,7 @@ as_util_status_csv (AsUtilPrivate *priv, gchar **values, GError **error)
 	_cleanup_string_free_ GString *data = NULL;
 
 	/* check args */
-	if (g_strv_length (values) != 2) {
+	if (g_strv_length (values) < 2) {
 		g_set_error_literal (error,
 				     AS_ERROR,
 				     AS_ERROR_INVALID_ARGUMENTS,
@@ -1720,6 +1738,19 @@ as_util_status_csv (AsUtilPrivate *priv, gchar **values, GError **error)
 		return FALSE;
 	}
 
+	/* process filters */
+	for (i = 2; values[i] != NULL; i++) {
+		_cleanup_strv_free_ gchar **split = NULL;
+		split = g_strsplit (values[i], "=", 2);
+		if (g_strv_length (split) != 2)
+			continue;
+		if (g_strcmp0 (split[0], "id-kind") == 0) {
+			id_kind = as_id_kind_from_string (split[1]);
+			continue;
+		}
+		g_warning ("Unknown filter option %s:%s", split[0], split[1]);
+	}
+
 	/* load file */
 	store = as_store_new ();
 	file = g_file_new_for_path (values[0]);
@@ -1732,13 +1763,7 @@ as_util_status_csv (AsUtilPrivate *priv, gchar **values, GError **error)
 	for (i = 0; i < apps->len; i++) {
 		_cleanup_free_ gchar *description = NULL;
 		app = g_ptr_array_index (apps, i);
-		if (as_app_get_id_kind (app) == AS_ID_KIND_FONT)
-			continue;
-		if (as_app_get_id_kind (app) == AS_ID_KIND_INPUT_METHOD)
-			continue;
-		if (as_app_get_id_kind (app) == AS_ID_KIND_CODEC)
-			continue;
-		if (as_app_get_id_kind (app) == AS_ID_KIND_SOURCE)
+		if (as_app_get_id_kind (app) != id_kind)
 			continue;
 		g_string_append_printf (data, "%s,", as_app_get_id (app));
 		g_string_append_printf (data, "%s,", as_app_get_pkgname_default (app));
@@ -2036,7 +2061,7 @@ as_util_check_root_app_icon (AsApp *app, GError **error)
 			     AS_ERROR,
 			     AS_ERROR_FAILED,
 			     "%s has no Icon",
-			     as_app_get_id_full (app));
+			     as_app_get_id (app));
 		return FALSE;
 	}
 
@@ -2051,7 +2076,7 @@ as_util_check_root_app_icon (AsApp *app, GError **error)
 	if (icon == NULL) {
 		g_prefix_error (error,
 				"%s missing icon %s: ",
-				as_app_get_id_full (app),
+				as_app_get_id (app),
 				as_app_get_icon (app));
 		return FALSE;
 	}
@@ -2061,7 +2086,7 @@ as_util_check_root_app_icon (AsApp *app, GError **error)
 	if (pb == NULL) {
 		g_prefix_error (error,
 				"%s invalid icon %s: ",
-				as_app_get_id_full (app),
+				as_app_get_id (app),
 				as_app_get_icon (app));
 		return FALSE;
 	}
@@ -2073,7 +2098,7 @@ as_util_check_root_app_icon (AsApp *app, GError **error)
 			     AS_ERROR,
 			     AS_ERROR_FAILED,
 			     "%s has undersized icon (%ix%i)",
-			     as_app_get_id_full (app),
+			     as_app_get_id (app),
 			     gdk_pixbuf_get_width (pb),
 			     gdk_pixbuf_get_height (pb));
 		return FALSE;
@@ -2103,7 +2128,7 @@ as_util_check_root_app (AsApp *app, GPtrArray *problems)
 	if (as_app_get_comment (app, NULL) == NULL) {
 		g_ptr_array_add (problems,
 				 g_strdup_printf ("%s has no Comment",
-						  as_app_get_id_full (app)));
+						  as_app_get_id (app)));
 	}
 
 	/* check icon exists and is large enough */
