@@ -158,8 +158,6 @@ as_app_validate_has_first_word_capital (AsAppValidateHelper *helper, const gchar
 	}
 
 	/* is the first word the project name */
-	if (g_strcmp0 (first_word, as_app_get_id (helper->app)) == 0)
-		return TRUE;
 	if (g_strcmp0 (first_word, as_app_get_name (helper->app, NULL)) == 0)
 		return TRUE;
 
@@ -180,7 +178,7 @@ as_app_validate_description_li (const gchar *text, AsAppValidateHelper *helper)
 	/* relax the requirements a bit */
 	if ((helper->flags & AS_APP_VALIDATE_FLAG_RELAX) > 0) {
 		length_li_max = 1000;
-		length_li_min = 4;
+		length_li_min = 3;
 		require_sentence_case = FALSE;
 	}
 
@@ -855,9 +853,13 @@ as_app_validate_license (const gchar *license_text, GError **error)
 
 	licenses = as_utils_spdx_license_tokenize (license_text);
 	for (i = 0; licenses[i] != NULL; i++) {
-		if (g_str_has_prefix (licenses[i], "#"))
+		if (g_strcmp0 (licenses[i], "&") == 0 ||
+		    g_strcmp0 (licenses[i], "|") == 0 ||
+		    g_strcmp0 (licenses[i], "(") == 0 ||
+		    g_strcmp0 (licenses[i], ")") == 0)
 			continue;
-		if (!as_utils_is_spdx_license_id (licenses[i])) {
+		if (licenses[i][0] != '@' ||
+		    !as_utils_is_spdx_license_id (licenses[i] + 1)) {
 			g_set_error (error,
 				     AS_APP_ERROR,
 				     AS_APP_ERROR_FAILED,
@@ -879,15 +881,15 @@ as_app_validate_is_content_license (const gchar *license)
 	_cleanup_strv_free_ gchar **tokens = NULL;
 	tokens = as_utils_spdx_license_tokenize (license);
 	for (i = 0; tokens[i] != NULL; i++) {
-		if (g_strcmp0 (tokens[i], "CC0-1.0") == 0)
+		if (g_strcmp0 (tokens[i], "@CC0-1.0") == 0)
 			continue;
-		if (g_strcmp0 (tokens[i], "CC-BY-3.0") == 0)
+		if (g_strcmp0 (tokens[i], "@CC-BY-3.0") == 0)
 			continue;
-		if (g_strcmp0 (tokens[i], "CC-BY-SA-3.0") == 0)
+		if (g_strcmp0 (tokens[i], "@CC-BY-SA-3.0") == 0)
 			continue;
-		if (g_strcmp0 (tokens[i], "GFDL-1.3") == 0)
+		if (g_strcmp0 (tokens[i], "@GFDL-1.3") == 0)
 			continue;
-		if (g_strcmp0 (tokens[i], "# and ") == 0)
+		if (g_strcmp0 (tokens[i], "&") == 0)
 			continue;
 		return FALSE;
 	}
@@ -916,7 +918,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	GList *l;
 	GPtrArray *probs = NULL;
 	const gchar *description;
-	const gchar *id_full;
+	const gchar *id;
 	const gchar *key;
 	const gchar *license;
 	const gchar *name;
@@ -981,26 +983,26 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 
 	/* id */
 	ret = FALSE;
-	id_full = as_app_get_id_full (app);
+	id = as_app_get_id (app);
 	switch (as_app_get_id_kind (app)) {
 	case AS_ID_KIND_DESKTOP:
-		if (g_str_has_suffix (id_full, ".desktop"))
+		if (g_str_has_suffix (id, ".desktop"))
 			ret = TRUE;
 		break;
 	case AS_ID_KIND_FONT:
-		if (g_str_has_suffix (id_full, ".ttf"))
+		if (g_str_has_suffix (id, ".ttf"))
 			ret = TRUE;
-		else if (g_str_has_suffix (id_full, ".otf"))
+		else if (g_str_has_suffix (id, ".otf"))
 			ret = TRUE;
 		break;
 	case AS_ID_KIND_INPUT_METHOD:
-		if (g_str_has_suffix (id_full, ".xml"))
+		if (g_str_has_suffix (id, ".xml"))
 			ret = TRUE;
-		else if (g_str_has_suffix (id_full, ".db"))
+		else if (g_str_has_suffix (id, ".db"))
 			ret = TRUE;
 		break;
 	case AS_ID_KIND_CODEC:
-		if (g_str_has_prefix (id_full, "gstreamer"))
+		if (g_str_has_prefix (id, "gstreamer"))
 			ret = TRUE;
 		break;
 	case AS_ID_KIND_UNKNOWN:
@@ -1140,7 +1142,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	}
 
 	/* check for things that have to exist */
-	if (as_app_get_id_full (app) == NULL) {
+	if (as_app_get_id (app) == NULL) {
 		ai_app_validate_add (probs,
 				     AS_PROBLEM_KIND_TAG_MISSING,
 				     "<id> is not present");
