@@ -36,6 +36,7 @@
 #include "as-store.h"
 #include "as-tag.h"
 #include "as-utils-private.h"
+#include "as-yaml.h"
 
 /**
  * cd_test_get_filename:
@@ -540,6 +541,9 @@ as_test_app_func (void)
 		"<kudos>"
 		"<kudo>SearchProvider</kudo>"
 		"</kudos>"
+		"<vetos>"
+		"<veto>Required AppData: ConsoleOnly</veto>"
+		"</vetos>"
 		"<mimetypes>"
 		"<mimetype>application/vnd.oasis.opendocument.spreadsheet</mimetype>"
 		"</mimetypes>"
@@ -586,8 +590,8 @@ as_test_app_func (void)
 	g_assert (ret);
 
 	/* verify */
-	g_assert_cmpstr (as_app_get_id_full (app), ==, "org.gnome.Software.desktop");
-	g_assert_cmpstr (as_app_get_id (app), ==, "org.gnome.Software");
+	g_assert_cmpstr (as_app_get_id (app), ==, "org.gnome.Software.desktop");
+	g_assert_cmpstr (as_app_get_id_filename (app), ==, "org.gnome.Software");
 	g_assert_cmpstr (as_app_get_name (app, "pl"), ==, "Oprogramowanie");
 	g_assert_cmpstr (as_app_get_comment (app, NULL), ==, "Application manager");
 	g_assert_cmpstr (as_app_get_developer_name (app, NULL), ==, "GNOME Foundation");
@@ -673,10 +677,10 @@ as_test_app_validate_appdata_good_func (void)
 
 	/* check success */
 	g_assert_cmpint (as_app_get_id_kind (app), ==, AS_ID_KIND_DESKTOP);
-	g_assert_cmpstr (as_app_get_id_full (app), ==, "gnome-power-statistics.desktop");
+	g_assert_cmpstr (as_app_get_id (app), ==, "gnome-power-statistics.desktop");
 	g_assert_cmpstr (as_app_get_name (app, "C"), ==, "0 A.D.");
 	g_assert_cmpstr (as_app_get_comment (app, "C"), ==, "Observe power management");
-	g_assert_cmpstr (as_app_get_metadata_license (app), ==, "CC0-1.0 and CC-BY-3.0");
+	g_assert_cmpstr (as_app_get_metadata_license (app), ==, "CC0-1.0 AND CC-BY-3.0");
 	g_assert_cmpstr (as_app_get_update_contact (app), ==, "richard@hughsie.com");
 	g_assert_cmpstr (as_app_get_project_group (app), ==, "GNOME");
 	g_assert_cmpstr (as_app_get_url_item (app, AS_URL_KIND_HOMEPAGE), ==,
@@ -688,7 +692,7 @@ as_test_app_validate_appdata_good_func (void)
 	g_assert (probs != NULL);
 	for (i = 0; i < probs->len; i++) {
 		problem = g_ptr_array_index (probs, i);
-		g_warning ("%s", as_problem_get_message (problem));
+		g_print ("%s\n", as_problem_get_message (problem));
 	}
 	g_assert_cmpint (probs->len, ==, 0);
 	g_ptr_array_unref (probs);
@@ -727,7 +731,7 @@ as_test_app_validate_metainfo_good_func (void)
 
 	/* check success */
 	g_assert_cmpint (as_app_get_id_kind (app), ==, AS_ID_KIND_ADDON);
-	g_assert_cmpstr (as_app_get_id_full (app), ==, "gedit-code-assistance");
+	g_assert_cmpstr (as_app_get_id (app), ==, "gedit-code-assistance");
 	g_assert_cmpstr (as_app_get_name (app, "C"), ==, "Code assistance");
 	g_assert_cmpstr (as_app_get_comment (app, "C"), ==, "Code assistance for C, C++ and Objective-C");
 	g_assert_cmpstr (as_app_get_metadata_license (app), ==, "CC0-1.0");
@@ -769,7 +773,7 @@ as_test_app_validate_intltool_func (void)
 
 	/* check success */
 	g_assert_cmpint (as_app_get_id_kind (app), ==, AS_ID_KIND_DESKTOP);
-	g_assert_cmpstr (as_app_get_id_full (app), ==, "gnome-power-statistics.desktop");
+	g_assert_cmpstr (as_app_get_id (app), ==, "gnome-power-statistics.desktop");
 	g_assert_cmpstr (as_app_get_name (app, "C"), ==, "0 A.D.");
 	g_assert_cmpstr (as_app_get_comment (app, "C"), ==, "Observe power management");
 	probs = as_app_validate (app, AS_APP_VALIDATE_FLAG_NO_NETWORK, &error);
@@ -1039,7 +1043,7 @@ as_test_app_validate_style_func (void)
 
 	app = as_app_new ();
 	as_app_add_url (app, AS_URL_KIND_UNKNOWN, "dave.com", -1);
-	as_app_set_id_full (app, "dave.exe", -1);
+	as_app_set_id (app, "dave.exe", -1);
 	as_app_set_id_kind (app, AS_ID_KIND_DESKTOP);
 	as_app_set_source_kind (app, AS_APP_SOURCE_KIND_APPDATA);
 	as_app_set_metadata_license (app, "BSD", -1);
@@ -1091,7 +1095,10 @@ as_test_app_parse_file_func (void)
 	/* create an AsApp from a desktop file */
 	app = as_app_new ();
 	filename = as_test_get_filename ("example.desktop");
-	ret = as_app_parse_file (app, filename, 0, &error);
+	ret = as_app_parse_file (app,
+				 filename,
+				 AS_APP_PARSE_FLAG_ALLOW_VETO,
+				 &error);
 	g_assert_no_error (error);
 	g_assert (ret);
 
@@ -1104,17 +1111,19 @@ as_test_app_parse_file_func (void)
 		"Badanie i porównywanie zainstalowanych profilów kolorów");
 	g_assert_cmpstr (as_app_get_icon (app), ==, "audio-input-microphone");
 	g_assert_cmpint (as_app_get_icon_kind (app), ==, AS_ICON_KIND_STOCK);
-	g_assert_cmpstr (as_app_get_metadata_item (app, "NoDisplay"), ==, "");
+	g_assert_cmpint (as_app_get_vetos(app)->len, ==, 1);
 	g_assert_cmpstr (as_app_get_project_group (app), ==, NULL);
 	g_assert_cmpstr (as_app_get_source_file (app), ==, filename);
 	g_assert_cmpint (as_app_get_categories(app)->len, ==, 1);
-	g_assert_cmpint (as_app_get_keywords(app)->len, ==, 2);
+	g_assert_cmpint (as_app_get_keywords(app, NULL)->len, ==, 2);
+	g_assert_cmpint (as_app_get_keywords(app, "pl")->len, ==, 1);
 	g_assert (as_app_has_category (app, "System"));
 	g_assert (!as_app_has_category (app, "NotGoingToExist"));
 
 	/* reparse with heuristics */
 	ret = as_app_parse_file (app,
 				 filename,
+				 AS_APP_PARSE_FLAG_ALLOW_VETO |
 				 AS_APP_PARSE_FLAG_USE_HEURISTICS,
 				 &error);
 	g_assert_no_error (error);
@@ -1160,7 +1169,7 @@ as_test_app_no_markup_func (void)
 	g_assert (ret);
 
 	/* verify */
-	g_assert_cmpstr (as_app_get_id_full (app), ==, "org.gnome.Software.desktop");
+	g_assert_cmpstr (as_app_get_id (app), ==, "org.gnome.Software.desktop");
 	g_assert_cmpstr (as_app_get_description (app, "C"), ==,
 		"Software is awesome:\n\n * Bada\n * Boom!");
 	as_node_unref (root);
@@ -1548,6 +1557,8 @@ as_test_app_subsume_func (void)
 	as_app_add_language (donor, -1, "en_GB", -1);
 	as_app_add_metadata (donor, "donor", "true", -1);
 	as_app_add_metadata (donor, "overwrite", "1111", -1);
+	as_app_add_keyword (donor, "C", "klass", -1);
+	as_app_add_keyword (donor, "pl", "klaski", -1);
 	ss = as_screenshot_new ();
 	as_app_add_screenshot (donor, ss);
 
@@ -1564,6 +1575,8 @@ as_test_app_subsume_func (void)
 	g_assert_cmpstr (as_app_get_metadata_item (donor, "recipient"), ==, NULL);
 	g_assert_cmpint (as_app_get_pkgnames(app)->len, ==, 1);
 	g_assert_cmpint (as_app_get_state (app), ==, AS_APP_STATE_INSTALLED);
+	g_assert_cmpint (as_app_get_keywords(app, "C")->len, ==, 1);
+	g_assert_cmpint (as_app_get_keywords(app, "pl")->len, ==, 1);
 	list = as_app_get_languages (app);
 	g_assert_cmpint (g_list_length (list), ==, 1);
 	g_list_free (list);
@@ -1589,10 +1602,12 @@ as_test_app_search_func (void)
 	as_app_set_name (app, NULL, "GNOME Software", -1);
 	as_app_set_comment (app, NULL, "Install and remove software", -1);
 	as_app_add_mimetype (app, "application/vnd.oasis.opendocument.text", -1);
+	as_app_add_keyword (app, NULL, "awesome", -1);
 
 	g_assert_cmpint (as_app_search_matches (app, "software"), ==, 80);
 	g_assert_cmpint (as_app_search_matches (app, "soft"), ==, 80);
 	g_assert_cmpint (as_app_search_matches (app, "install"), ==, 60);
+	g_assert_cmpint (as_app_search_matches (app, "awesome"), ==, 90);
 	g_assert_cmpint (as_app_search_matches_all (app, (gchar**) all), ==, 220);
 	g_assert_cmpint (as_app_search_matches_all (app, (gchar**) none), ==, 0);
 	g_assert_cmpint (as_app_search_matches_all (app, (gchar**) mime), ==, 5);
@@ -1605,7 +1620,8 @@ as_test_store_demote_func (void)
 	AsApp *app;
 	GError *error = NULL;
 	gboolean ret;
-	_cleanup_free_ gchar *filename = NULL;
+	_cleanup_free_ gchar *filename1 = NULL;
+	_cleanup_free_ gchar *filename2 = NULL;
 	_cleanup_object_unref_ AsApp *app_appdata = NULL;
 	_cleanup_object_unref_ AsApp *app_desktop = NULL;
 	_cleanup_object_unref_ AsStore *store = NULL;
@@ -1613,8 +1629,8 @@ as_test_store_demote_func (void)
 
 	/* load example desktop file */
 	app_desktop = as_app_new ();
-	filename = as_test_get_filename ("example.desktop");
-	ret = as_app_parse_file (app_desktop, filename,
+	filename1 = as_test_get_filename ("example.desktop");
+	ret = as_app_parse_file (app_desktop, filename1,
 				 AS_APP_PARSE_FLAG_ALLOW_VETO, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
@@ -1622,8 +1638,8 @@ as_test_store_demote_func (void)
 
 	/* load example appdata file */
 	app_appdata = as_app_new ();
-	filename = as_test_get_filename ("example.appdata.xml");
-	ret = as_app_parse_file (app_appdata, filename,
+	filename2 = as_test_get_filename ("example.appdata.xml");
+	ret = as_app_parse_file (app_appdata, filename2,
 				 AS_APP_PARSE_FLAG_ALLOW_VETO, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
@@ -1663,7 +1679,7 @@ as_test_store_merges_func (void)
 	store_desktop_appdata = as_store_new ();
 
 	app_desktop = as_app_new ();
-	as_app_set_id_full (app_desktop, "gimp.desktop", -1);
+	as_app_set_id (app_desktop, "gimp.desktop", -1);
 	as_app_set_source_kind (app_desktop, AS_APP_SOURCE_KIND_DESKTOP);
 	as_app_set_name (app_desktop, NULL, "GIMP", -1);
 	as_app_set_comment (app_desktop, NULL, "GNU Bla Bla", -1);
@@ -1671,7 +1687,7 @@ as_test_store_merges_func (void)
 	as_app_set_state (app_desktop, AS_APP_STATE_INSTALLED);
 
 	app_appdata = as_app_new ();
-	as_app_set_id_full (app_appdata, "gimp.desktop", -1);
+	as_app_set_id (app_appdata, "gimp.desktop", -1);
 	as_app_set_source_kind (app_appdata, AS_APP_SOURCE_KIND_APPDATA);
 	as_app_set_description (app_appdata, NULL, "<p>Gimp is awesome</p>", -1);
 	as_app_add_pkgname (app_appdata, "gimp", -1);
@@ -1694,7 +1710,7 @@ as_test_store_merges_func (void)
 	store_all = as_store_new ();
 
 	app_appinfo = as_app_new ();
-	as_app_set_id_full (app_appinfo, "gimp.desktop", -1);
+	as_app_set_id (app_appinfo, "gimp.desktop", -1);
 	as_app_set_source_kind (app_appinfo, AS_APP_SOURCE_KIND_APPSTREAM);
 	as_app_set_name (app_appinfo, NULL, "GIMP", -1);
 	as_app_set_comment (app_appinfo, NULL, "GNU Bla Bla", -1);
@@ -1731,7 +1747,7 @@ as_test_store_merges_local_func (void)
 	as_store_set_add_flags (store, AS_STORE_ADD_FLAG_PREFER_LOCAL);
 
 	app_desktop = as_app_new ();
-	as_app_set_id_full (app_desktop, "gimp.desktop", -1);
+	as_app_set_id (app_desktop, "gimp.desktop", -1);
 	as_app_set_source_kind (app_desktop, AS_APP_SOURCE_KIND_DESKTOP);
 	as_app_set_name (app_desktop, NULL, "GIMP", -1);
 	as_app_set_comment (app_desktop, NULL, "GNU Bla Bla", -1);
@@ -1739,7 +1755,7 @@ as_test_store_merges_local_func (void)
 	as_app_set_state (app_desktop, AS_APP_STATE_INSTALLED);
 
 	app_appdata = as_app_new ();
-	as_app_set_id_full (app_appdata, "gimp.desktop", -1);
+	as_app_set_id (app_appdata, "gimp.desktop", -1);
 	as_app_set_source_kind (app_appdata, AS_APP_SOURCE_KIND_APPDATA);
 	as_app_set_description (app_appdata, NULL, "<p>Gimp is awesome</p>", -1);
 	as_app_add_pkgname (app_appdata, "gimp", -1);
@@ -1747,7 +1763,7 @@ as_test_store_merges_local_func (void)
 	as_app_set_state (app_appdata, AS_APP_STATE_INSTALLED);
 
 	app_appinfo = as_app_new ();
-	as_app_set_id_full (app_appinfo, "gimp.desktop", -1);
+	as_app_set_id (app_appinfo, "gimp.desktop", -1);
 	as_app_set_source_kind (app_appinfo, AS_APP_SOURCE_KIND_APPSTREAM);
 	as_app_set_name (app_appinfo, NULL, "GIMP", -1);
 	as_app_set_comment (app_appinfo, NULL, "Fedora GNU Bla Bla", -1);
@@ -1783,7 +1799,7 @@ as_test_store_func (void)
 	g_assert_cmpfloat (as_store_get_api_version (store), <, 1.f);
 	g_assert_cmpfloat (as_store_get_api_version (store), >, 0.f);
 	app = as_app_new ();
-	as_app_set_id_full (app, "gnome-software.desktop", -1);
+	as_app_set_id (app, "gnome-software.desktop", -1);
 	as_app_set_id_kind (app, AS_ID_KIND_DESKTOP);
 	as_store_add_app (store, app);
 	g_object_unref (app);
@@ -1791,7 +1807,7 @@ as_test_store_func (void)
 
 	/* add and then remove another app */
 	app = as_app_new ();
-	as_app_set_id_full (app, "junk.desktop", -1);
+	as_app_set_id (app, "junk.desktop", -1);
 	as_app_set_id_kind (app, AS_ID_KIND_FONT);
 	as_store_add_app (store, app);
 	g_object_unref (app);
@@ -1810,7 +1826,7 @@ as_test_store_func (void)
 
 	/* add another app and ensure it's sorted */
 	app = as_app_new ();
-	as_app_set_id_full (app, "aaa.desktop", -1);
+	as_app_set_id (app, "aaa.desktop", -1);
 	as_app_set_id_kind (app, AS_ID_KIND_FONT);
 	as_store_add_app (store, app);
 	g_object_unref (app);
@@ -1937,9 +1953,6 @@ as_test_store_addons_func (void)
 	gboolean ret;
 	const gchar *xml =
 		"<components version=\"0.7\">"
-		"<component type=\"desktop\">"
-		"<id>eclipse.desktop</id>"
-		"</component>"
 		"<component type=\"addon\">"
 		"<id>eclipse-php.jar</id>"
 		"<mimetypes>"
@@ -1947,6 +1960,9 @@ as_test_store_addons_func (void)
 		"</mimetypes>"
 		"<extends>eclipse.desktop</extends>"
 		"</component>"
+		"<component type=\"desktop\">"
+		"<id>eclipse.desktop</id>"
+		"</component>"
 		"</components>";
 	_cleanup_object_unref_ AsStore *store;
 	_cleanup_string_free_ GString *str;
@@ -1969,13 +1985,15 @@ as_test_store_addons_func (void)
 	data = as_app_get_addons (app);
 	g_assert_cmpint (data->len, ==, 1);
 	app = g_ptr_array_index (data, 0);
-	g_assert_cmpstr (as_app_get_id_full (app), ==, "eclipse-php.jar");
+	g_assert_cmpstr (as_app_get_id (app), ==, "eclipse-php.jar");
 
 	/* check we can search for token from the addon */
 	g_assert_cmpint (as_app_search_matches (app, "xtest"), >, 0);
 
 	/* check it marshals back to the same XML */
 	str = as_store_to_xml (store, 0);
+	if (g_strcmp0 (str->str, xml) != 0)
+		g_warning ("Expected:\n%s\nGot:\n%s", xml, str->str);
 	g_assert_cmpstr (str->str, ==, xml);
 }
 
@@ -2190,71 +2208,86 @@ as_test_utils_spdx_token_func (void)
 	gchar *tmp;
 
 	/* simple */
-	tok = as_utils_spdx_license_tokenize ("GPL");
-	tmp = g_strjoinv ("|", tok);
-	g_assert_cmpstr (tmp, ==, "GPL");
+	tok = as_utils_spdx_license_tokenize ("LGPL-2.0+");
+	tmp = g_strjoinv ("  ", tok);
+	g_assert_cmpstr (tmp, ==, "@LGPL-2.0+");
 	g_strfreev (tok);
 	g_free (tmp);
 
 	/* empty */
 	tok = as_utils_spdx_license_tokenize ("");
-	tmp = g_strjoinv ("|", tok);
+	tmp = g_strjoinv ("  ", tok);
 	g_assert_cmpstr (tmp, ==, "");
 	g_strfreev (tok);
 	g_free (tmp);
 
-	/* multiple licences */
-	tok = as_utils_spdx_license_tokenize ("GPL and MPL and CDL");
-	tmp = g_strjoinv ("|", tok);
-	g_assert_cmpstr (tmp, ==, "GPL|# and |MPL|# and |CDL");
+	/* random */
+	tok = as_utils_spdx_license_tokenize ("Public Domain");
+	tmp = g_strjoinv ("  ", tok);
+	g_assert_cmpstr (tmp, ==, "Public Domain");
 	g_strfreev (tok);
 	g_free (tmp);
 
 	/* multiple licences */
-	tok = as_utils_spdx_license_tokenize ("GPL and MPL or BSD and MPL");
-	tmp = g_strjoinv ("|", tok);
-	g_assert_cmpstr (tmp, ==, "GPL|# and |MPL|# or |BSD|# and |MPL");
+	tok = as_utils_spdx_license_tokenize ("LGPL-2.0+ AND GPL-2.0 AND LGPL-3.0");
+	tmp = g_strjoinv ("  ", tok);
+	g_assert_cmpstr (tmp, ==, "@LGPL-2.0+  &  @GPL-2.0  &  @LGPL-3.0");
+	g_strfreev (tok);
+	g_free (tmp);
+
+	/* multiple licences, deprectated 'and' & 'or' */
+	tok = as_utils_spdx_license_tokenize ("LGPL-2.0+ and GPL-2.0 or LGPL-3.0");
+	tmp = g_strjoinv ("  ", tok);
+	g_assert_cmpstr (tmp, ==, "@LGPL-2.0+  &  @GPL-2.0  |  @LGPL-3.0");
 	g_strfreev (tok);
 	g_free (tmp);
 
 	/* brackets */
-	tok = as_utils_spdx_license_tokenize ("LGPLv2+ and (QPL or GPLv2) and MIT");
-	tmp = g_strjoinv ("|", tok);
-	g_assert_cmpstr (tmp, ==, "LGPLv2+|# and (|QPL|# or |GPLv2|#) and |MIT");
+	tok = as_utils_spdx_license_tokenize ("LGPL-2.0+ and (GPL-2.0 or GPL-2.0+) and MIT");
+	tmp = g_strjoinv ("  ", tok);
+	g_assert_cmpstr (tmp, ==, "@LGPL-2.0+  &  (  @GPL-2.0  |  @GPL-2.0+  )  &  @MIT");
 	g_strfreev (tok);
 	g_free (tmp);
 
 	/* detokenisation */
 	tok = as_utils_spdx_license_tokenize ("LGPLv2+ and (QPL or GPLv2) and MIT");
 	tmp = as_utils_spdx_license_detokenize (tok);
-	g_assert_cmpstr (tmp, ==, "LGPLv2+ and (QPL or GPLv2) and MIT");
+	g_assert_cmpstr (tmp, ==, "LGPLv2+ AND (QPL OR GPLv2) AND MIT");
+	g_strfreev (tok);
+	g_free (tmp);
+
+	/* detokenisation literals */
+	tok = as_utils_spdx_license_tokenize ("Public Domain");
+	tmp = as_utils_spdx_license_detokenize (tok);
+	g_assert_cmpstr (tmp, ==, "Public Domain");
 	g_strfreev (tok);
 	g_free (tmp);
 
 	/* leading brackets */
 	tok = as_utils_spdx_license_tokenize ("(MPLv1.1 or LGPLv3+) and LGPLv3");
-	tmp = g_strjoinv ("|", tok);
-	g_assert_cmpstr (tmp, ==, "#(|MPLv1.1|# or |LGPLv3+|#) and |LGPLv3");
+	tmp = g_strjoinv ("  ", tok);
+	g_assert_cmpstr (tmp, ==, "(  MPLv1.1  |  LGPLv3+  )  &  LGPLv3");
 	g_strfreev (tok);
 	g_free (tmp);
 
 	/*  trailing brackets */
 	tok = as_utils_spdx_license_tokenize ("MPLv1.1 and (LGPLv3 or GPLv3)");
-	tmp = g_strjoinv ("|", tok);
-	g_assert_cmpstr (tmp, ==, "MPLv1.1|# and (|LGPLv3|# or |GPLv3|#)");
+	tmp = g_strjoinv ("  ", tok);
+	g_assert_cmpstr (tmp, ==, "MPLv1.1  &  (  LGPLv3  |  GPLv3  )");
 	g_strfreev (tok);
 	g_free (tmp);
 
 	/*  deprecated names */
 	tok = as_utils_spdx_license_tokenize ("CC0 and (CC0 or CC0)");
-	tmp = g_strjoinv ("|", tok);
-	g_assert_cmpstr (tmp, ==, "CC0-1.0|# and (|CC0-1.0|# or |CC0-1.0|#)");
+	tmp = g_strjoinv ("  ", tok);
+	g_assert_cmpstr (tmp, ==, "@CC0-1.0  &  (  @CC0-1.0  |  @CC0-1.0  )");
 	g_strfreev (tok);
 	g_free (tmp);
 
 	/* SPDX strings */
 	g_assert (as_utils_is_spdx_license ("CC0"));
 	g_assert (as_utils_is_spdx_license ("CC0 and GFDL-1.3"));
+	g_assert (as_utils_is_spdx_license ("CC0 AND GFDL-1.3"));
 	g_assert (!as_utils_is_spdx_license ("CC0 dave"));
 }
 
@@ -2387,6 +2420,246 @@ as_test_store_metadata_func (void)
 	g_ptr_array_unref (apps);
 }
 
+static void
+as_test_store_metadata_index_func (void)
+{
+	GPtrArray *apps;
+	const guint repeats = 10000;
+	guint i;
+	_cleanup_object_unref_ AsStore *store = NULL;
+	_cleanup_timer_destroy_ GTimer *timer = NULL;
+
+	/* create lots of applications in the store */
+	store = as_store_new ();
+	as_store_add_metadata_index (store, "X-CacheID");
+	for (i = 0; i < repeats; i++) {
+		_cleanup_free_ gchar *id = g_strdup_printf ("app-%05i", i);
+		_cleanup_object_unref_ AsApp *app = as_app_new ();
+		as_app_set_id (app, id, -1);
+		as_app_add_metadata (app, "X-CacheID", "dave.i386", -1);
+		as_app_add_metadata (app, "baz", "dave", -1);
+		as_store_add_app (store, app);
+	}
+
+	/* find out how long this takes with an index */
+	timer = g_timer_new ();
+	for (i = 0; i < repeats; i++) {
+		apps = as_store_get_apps_by_metadata (store, "X-CacheID", "dave.i386");
+		g_assert_cmpint (apps->len, ==, repeats);
+		g_ptr_array_unref (apps);
+		apps = as_store_get_apps_by_metadata (store, "X-CacheID", "notgoingtoexist");
+		g_assert_cmpint (apps->len, ==, 0);
+		g_ptr_array_unref (apps);
+	}
+	g_assert_cmpfloat (g_timer_elapsed (timer, NULL), <, 0.5);
+	g_print ("%.0fms: ", g_timer_elapsed (timer, NULL) * 1000);
+}
+
+static void
+as_test_yaml_func (void)
+{
+	GNode *node;
+	GError *error = NULL;
+	GString *str;
+	const gchar *expected;
+	_cleanup_free_ gchar *filename = NULL;
+	_cleanup_object_unref_ GFile *file = NULL;
+
+	/* simple header */
+	node = as_yaml_from_data (
+		"File: DEP-11\n"
+		"Origin: aequorea\n"
+		"Version: '0.6'\n",
+		-1, &error);
+	g_assert_no_error (error);
+	g_assert (node != NULL);
+	str = as_yaml_to_string (node);
+	expected =
+		"[MAP]{\n"
+		" [KVL]File=DEP-11\n"
+		" [KVL]Origin=aequorea\n"
+		" [KVL]Version=0.6\n";
+	if (g_strcmp0 (str->str, expected) != 0)
+		g_warning ("Expected:\n%s\nGot:\n%s", expected, str->str);
+	g_assert_cmpstr (str->str, ==, expected);
+	g_string_free (str, TRUE);
+	as_yaml_unref (node);
+
+	/* simple list */
+	node = as_yaml_from_data (
+		"Mimetypes:\n"
+		"  - text/html\n"
+		"  - text/xml\n"
+		"  - application/xhtml+xml\n"
+		"Kudos:\n"
+		"  - AppMenu\n"
+		"  - SearchProvider\n"
+		"  - Notifications\n",
+		-1, &error);
+	g_assert_no_error (error);
+	g_assert (node != NULL);
+	str = as_yaml_to_string (node);
+	expected =
+		"[MAP]{\n"
+		" [SEQ]Mimetypes\n"
+		"  [KEY]text/html\n"
+		"  [KEY]text/xml\n"
+		"  [KEY]application/xhtml+xml\n"
+		" [SEQ]Kudos\n"
+		"  [KEY]AppMenu\n"
+		"  [KEY]SearchProvider\n"
+		"  [KEY]Notifications\n";
+	if (g_strcmp0 (str->str, expected) != 0)
+		g_warning ("Expected:\n%s\nGot:\n%s", expected, str->str);
+	g_assert_cmpstr (str->str, ==, expected);
+	g_string_free (str, TRUE);
+	as_yaml_unref (node);
+
+	/* dummy application */
+	filename = as_test_get_filename ("example.yml");
+	g_assert (filename != NULL);
+	file = g_file_new_for_path (filename);
+	node = as_yaml_from_file (file, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (node != NULL);
+	str = as_yaml_to_string (node);
+	expected =
+		"[MAP]{\n"
+		" [KVL]File=DEP-11\n"
+		" [KVL]Origin=aequorea\n"
+		" [KVL]Version=0.6\n"
+		"[MAP]{\n"
+		" [KVL]Type=desktop-app\n"
+		" [KVL]ID=iceweasel.desktop\n"
+		" [MAP]Name\n"
+		"  [KVL]C=Iceweasel\n"
+		" [SEQ]Packages\n"
+		"  [KEY]iceweasel\n"
+		" [MAP]Icon\n"
+		"  [KVL]cached=iceweasel.png\n"
+		" [MAP]Keywords\n"
+		"  [SEQ]C\n"
+		"   [KEY]browser\n"
+		" [SEQ]Screenshots\n"
+		"  [MAP]{\n"
+		"   [KVL]default=true\n"
+		"   [MAP]source-image\n"
+		"    [KVL]height=770\n"
+		"    [KVL]url=http://localhost/source/screenshot.png\n"
+		"    [KVL]width=1026\n"
+		"   [SEQ]thumbnails\n"
+		"    [MAP]{\n"
+		"     [KVL]height=423\n"
+		"     [KVL]url=http://localhost/752x423/screenshot.png\n"
+		"     [KVL]width=752\n"
+		"[MAP]{\n"
+		" [KVL]Type=desktop-app\n"
+		" [KVL]ID=dave.desktop\n"
+		" [MAP]Name\n"
+		"  [KVL]C=dave\n";
+	if (g_strcmp0 (str->str, expected) != 0)
+		g_warning ("Expected:\n%s\nGot:\n%s", expected, str->str);
+	g_assert_cmpstr (str->str, ==, expected);
+	g_string_free (str, TRUE);
+	as_yaml_unref (node);
+
+}
+
+static void
+as_test_store_yaml_func (void)
+{
+	AsApp *app;
+	GError *error = NULL;
+	gboolean ret;
+	_cleanup_free_ gchar *filename = NULL;
+	_cleanup_object_unref_ AsStore *store = NULL;
+	_cleanup_object_unref_ GFile *file = NULL;
+	_cleanup_string_free_ GString *str = NULL;
+	const gchar *xml =
+		"<components version=\"0.6\" origin=\"aequorea\">\n"
+		"<component type=\"desktop\">\n"
+		"<id>dave.desktop</id>\n"
+		"<name>dave</name>\n"
+		"</component>\n"
+		"<component type=\"desktop\">\n"
+		"<id>iceweasel.desktop</id>\n"
+		"<pkgname>iceweasel</pkgname>\n"
+		"<name>Iceweasel</name>\n"
+		"<icon type=\"cached\">iceweasel.png</icon>\n"
+		"<keywords>\n"
+		"<keyword>browser</keyword>\n"
+		"</keywords>\n"
+		"<screenshots>\n"
+		"<screenshot type=\"default\">\n"
+		"<image type=\"source\" height=\"770\" width=\"1026\">http://localhost/source/screenshot.png</image>\n"
+		"<image type=\"thumbnail\" height=\"423\" width=\"752\">http://localhost/752x423/screenshot.png</image>\n"
+		"</screenshot>\n"
+		"</screenshots>\n"
+		"</component>\n"
+		"</components>\n";
+
+	/* load store */
+	store = as_store_new ();
+	filename = as_test_get_filename ("example.yml");
+	file = g_file_new_for_path (filename);
+	ret = as_store_from_file (store, file, NULL, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* test it matches expected XML */
+	str = as_store_to_xml (store, AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE);
+	if (g_strcmp0 (str->str, xml) != 0)
+		g_warning ("Expected:\n%s\nGot:\n%s", xml, str->str);
+	g_assert_cmpstr (str->str, ==, xml);
+
+	/* test store properties */
+	g_assert_cmpstr (as_store_get_origin (store), ==, "aequorea");
+	g_assert_cmpfloat (as_store_get_api_version (store), <, 0.6 + 0.01);
+	g_assert_cmpfloat (as_store_get_api_version (store), >, 0.6 - 0.01);
+	g_assert_cmpint (as_store_get_size (store), ==, 2);
+	g_assert (as_store_get_app_by_id (store, "iceweasel.desktop") != NULL);
+	g_assert (as_store_get_app_by_id (store, "dave.desktop") != NULL);
+
+	/* test application properties */
+	app = as_store_get_app_by_id (store, "iceweasel.desktop");
+	g_assert_cmpint (as_app_get_id_kind (app), ==, AS_ID_KIND_DESKTOP);
+	g_assert_cmpstr (as_app_get_pkgname_default (app), ==, "iceweasel");
+	g_assert_cmpstr (as_app_get_name (app, "C"), ==, "Iceweasel");
+}
+
+static void
+as_test_store_speed_yaml_func (void)
+{
+	GError *error = NULL;
+	gboolean ret;
+	guint i;
+	guint loops = 10;
+	_cleanup_free_ gchar *filename = NULL;
+	_cleanup_object_unref_ GFile *file = NULL;
+	_cleanup_timer_destroy_ GTimer *timer = NULL;
+
+	filename = as_test_get_filename ("example-v06.yml.gz");
+	g_assert (filename != NULL);
+	file = g_file_new_for_path (filename);
+	timer = g_timer_new ();
+	for (i = 0; i < loops; i++) {
+		_cleanup_object_unref_ AsStore *store;
+		store = as_store_new ();
+		ret = as_store_from_file (store, file, NULL, NULL, &error);
+		g_assert_no_error (error);
+		g_assert (ret);
+
+		/* test store properties */
+		g_assert_cmpstr (as_store_get_origin (store), ==, "bartholomea");
+		g_assert_cmpfloat (as_store_get_api_version (store), <, 0.6 + 0.01);
+		g_assert_cmpfloat (as_store_get_api_version (store), >, 0.6 - 0.01);
+		g_assert_cmpint (as_store_get_size (store), ==, 85);
+		g_assert (as_store_get_app_by_id (store, "blobwars.desktop") != NULL);
+	}
+	g_print ("%.0f ms: ", g_timer_elapsed (timer, NULL) * 1000 / loops);
+
+}
+
 int
 main (int argc, char **argv)
 {
@@ -2429,6 +2702,7 @@ main (int argc, char **argv)
 	g_test_add_func ("/AppStream/utils", as_test_utils_func);
 	g_test_add_func ("/AppStream/utils{icons}", as_test_utils_icons_func);
 	g_test_add_func ("/AppStream/utils{spdx-token}", as_test_utils_spdx_token_func);
+	g_test_add_func ("/AppStream/yaml", as_test_yaml_func);
 	g_test_add_func ("/AppStream/store", as_test_store_func);
 	g_test_add_func ("/AppStream/store{demote}", as_test_store_demote_func);
 	g_test_add_func ("/AppStream/store{merges}", as_test_store_merges_func);
@@ -2437,13 +2711,16 @@ main (int argc, char **argv)
 	g_test_add_func ("/AppStream/store{versions}", as_test_store_versions_func);
 	g_test_add_func ("/AppStream/store{origin}", as_test_store_origin_func);
 	g_test_add_func ("/AppStream/store{app-install}", as_test_store_app_install_func);
+	g_test_add_func ("/AppStream/store{yaml}", as_test_store_yaml_func);
 	g_test_add_func ("/AppStream/store{metadata}", as_test_store_metadata_func);
+	g_test_add_func ("/AppStream/store{metadata-index}", as_test_store_metadata_index_func);
 	g_test_add_func ("/AppStream/store{validate}", as_test_store_validate_func);
 	g_test_add_func ("/AppStream/store{local-app-install}", as_test_store_local_app_install_func);
 	g_test_add_func ("/AppStream/store{local-appdata}", as_test_store_local_appdata_func);
 	g_test_add_func ("/AppStream/store{speed-appstream}", as_test_store_speed_appstream_func);
 	g_test_add_func ("/AppStream/store{speed-appdata}", as_test_store_speed_appdata_func);
 	g_test_add_func ("/AppStream/store{speed-desktop}", as_test_store_speed_desktop_func);
+	g_test_add_func ("/AppStream/store{speed-yaml}", as_test_store_speed_yaml_func);
 
 	return g_test_run ();
 }
