@@ -42,6 +42,7 @@
 #include "as-problem.h"
 #include "as-store.h"
 #include "as-utils-private.h"
+#include "as-yaml.h"
 
 #define AS_API_VERSION_NEWEST	0.6
 
@@ -59,9 +60,10 @@ struct _AsStorePrivate
 	gchar			*builder_id;
 	gdouble			 api_version;
 	GPtrArray		*array;		/* of AsApp */
-	GHashTable		*hash_id;	/* of AsApp{id_full} */
+	GHashTable		*hash_id;	/* of AsApp{id} */
 	GHashTable		*hash_pkgname;	/* of AsApp{pkgname} */
 	GPtrArray		*file_monitors;	/* of GFileMonitor */
+	GHashTable		*metadata_indexes;	/* GHashTable{key} */
 	AsStoreAddFlags		 add_flags;
 	AsStoreProblems		 problems;
 };
@@ -109,6 +111,7 @@ as_store_finalize (GObject *object)
 	g_ptr_array_unref (priv->file_monitors);
 	g_hash_table_unref (priv->hash_id);
 	g_hash_table_unref (priv->hash_pkgname);
+	g_hash_table_unref (priv->metadata_indexes);
 
 	G_OBJECT_CLASS (as_store_parent_class)->finalize (object);
 }
@@ -131,6 +134,10 @@ as_store_init (AsStore *store)
 						    g_free,
 						    (GDestroyNotify) g_object_unref);
 	priv->file_monitors = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
+	priv->metadata_indexes = g_hash_table_new_full (g_str_hash,
+							  g_str_equal,
+							  g_free,
+							  (GDestroyNotify) g_hash_table_unref);
 }
 
 /**
@@ -215,6 +222,46 @@ as_store_remove_all (AsStore *store)
 	g_hash_table_remove_all (priv->hash_pkgname);
 }
 
+/**
+ * as_store_regen_metadata_index_key:
+ **/
+static void
+as_store_regen_metadata_index_key (AsStore *store, const gchar *key)
+{
+	AsApp *app;
+	AsStorePrivate *priv = GET_PRIVATE (store);
+	GHashTable *md;
+	GPtrArray *apps;
+	const gchar *tmp;
+	guint i;
+
+	/* regenerate cache */
+	md = g_hash_table_new_full (g_str_hash, g_str_equal,
+				    g_free, (GDestroyNotify) g_ptr_array_unref);
+	for (i = 0; i < priv->array->len; i++) {
+		app = g_ptr_array_index (priv->array, i);
+
+		/* no data */
+		tmp = as_app_get_metadata_item (app, key);
+		if (tmp == NULL)
+			continue;
+
+		/* seen before */
+		apps = g_hash_table_lookup (md, tmp);
+		if (apps != NULL) {
+			g_ptr_array_add (apps, g_object_ref (app));
+			continue;
+		}
+
+		/* never seen before */
+		apps = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
+		g_ptr_array_add (apps, g_object_ref (app));
+		g_hash_table_insert (md, g_strdup (tmp), apps);
+
+	}
+	g_hash_table_insert (priv->metadata_indexes, g_strdup (key), md);
+}
+
 /**
  * as_store_get_apps_by_metadata:
  * @store: a #AsStore instance.
@@ -234,11 +281,25 @@ as_store_get_apps_by_metadata (AsStore *store,
 {
 	AsApp *app;
 	AsStorePrivate *priv = GET_PRIVATE (store);
+	GHashTable *index;
 	GPtrArray *apps;
 	guint i;
 
 	g_return_val_if_fail (AS_IS_STORE (store), NULL);
 
+	/* do we have this indexed? */
+	index = g_hash_table_lookup (priv->metadata_indexes, key);
+	if (index != NULL) {
+		if (g_hash_table_size (index) == 0) {
+			as_store_regen_metadata_index_key (store, key);
+			index = g_hash_table_lookup (priv->metadata_indexes, key);
+		}
+		apps = g_hash_table_lookup (index, value);
+		if (apps != NULL)
+			return g_ptr_array_ref (apps);
+		return g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
+	}
+
 	/* find all the apps with this specific metadata key */
 	apps = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	for (i = 0; i < priv->array->len; i++) {
@@ -250,6 +311,26 @@ as_store_get_apps_by_metadata (AsStore *store,
 	return apps;
 }
 
+/**
+ * as_store_add_metadata_index:
+ * @store: a #AsStore instance.
+ * @key: the metadata key.
+ *
+ * Adds a metadata index key.
+ *
+ * NOTE: if applications are removed *all* the indexes will be invalid and
+ * will have to be re-added.
+ *
+ * Returns: (transfer none): a #AsApp or %NULL
+ *
+ * Since: 0.3.0
+ **/
+void
+as_store_add_metadata_index (AsStore *store, const gchar *key)
+{
+	as_store_regen_metadata_index_key (store, key);
+}
+
 /**
  * as_store_get_app_by_id:
  * @store: a #AsStore instance.
@@ -301,8 +382,36 @@ void
 as_store_remove_app (AsStore *store, AsApp *app)
 {
 	AsStorePrivate *priv = GET_PRIVATE (store);
-	g_hash_table_remove (priv->hash_id, as_app_get_id_full (app));
+	g_hash_table_remove (priv->hash_id, as_app_get_id (app));
 	g_ptr_array_remove (priv->array, app);
+	g_hash_table_remove_all (priv->metadata_indexes);
+}
+
+/**
+ * as_store_remove_app_by_id:
+ * @store: a #AsStore instance.
+ * @id: an application id
+ *
+ * Removes an application from the store if it exists.
+ *
+ * Since: 0.3.0
+ **/
+void
+as_store_remove_app_by_id (AsStore *store, const gchar *id)
+{
+	AsApp *app;
+	AsStorePrivate *priv = GET_PRIVATE (store);
+	guint i;
+
+	if (!g_hash_table_remove (priv->hash_id, id))
+		return;
+	for (i = 0; i < priv->array->len; i++) {
+		app = g_ptr_array_index (priv->array, i);
+		if (g_strcmp0 (id, as_app_get_id (app)) != 0)
+			continue;
+		g_ptr_array_remove (priv->array, app);
+	}
+	g_hash_table_remove_all (priv->metadata_indexes);
 }
 
 /**
@@ -328,7 +437,7 @@ as_store_add_app (AsStore *store, AsApp *app)
 	guint i;
 
 	/* have we recorded this before? */
-	id = as_app_get_id_full (app);
+	id = as_app_get_id (app);
 	if (id == NULL) {
 		g_warning ("application has no ID set");
 		return;
@@ -418,7 +527,7 @@ as_store_add_app (AsStore *store, AsApp *app)
 	/* success, add to array */
 	g_ptr_array_add (priv->array, g_object_ref (app));
 	g_hash_table_insert (priv->hash_id,
-			     (gpointer) as_app_get_id_full (app),
+			     (gpointer) as_app_get_id (app),
 			     app);
 	pkgnames = as_app_get_pkgnames (app);
 	for (i = 0; i < pkgnames->len; i++) {
@@ -450,7 +559,7 @@ as_store_match_addons (AsStore *store)
 		plugin_ids = as_app_get_extends (app);
 		if (plugin_ids->len == 0) {
 			g_warning ("%s was of type addon but had no extends",
-				   as_app_get_id_full (app));
+				   as_app_get_id (app));
 			continue;
 		}
 		for (j = 0; j < plugin_ids->len; j++) {
@@ -542,6 +651,53 @@ as_store_from_root (AsStore *store,
 	return TRUE;
 }
 
+/**
+ * as_store_load_yaml_file:
+ **/
+static gboolean
+as_store_load_yaml_file (AsStore *store,
+			 GFile *file,
+			 GCancellable *cancellable,
+			 GError **error)
+{
+	GNode *app_n;
+	GNode *n;
+	const gchar *tmp;
+	_cleanup_yaml_unref_ GNode *root = NULL;
+
+	/* load file */
+	root = as_yaml_from_file (file, cancellable, error);
+	if (root == NULL)
+		return FALSE;
+
+	/* get header information */
+	for (n = root->children->children; n != NULL; n = n->next) {
+		tmp = as_yaml_node_get_key (n);
+		if (g_strcmp0 (tmp, "Origin") == 0) {
+			as_store_set_origin (store, as_yaml_node_get_value (n));
+			continue;
+		}
+		if (g_strcmp0 (tmp, "Version") == 0) {
+			if (as_yaml_node_get_value (n) != NULL)
+				as_store_set_api_version (store, g_ascii_strtod (as_yaml_node_get_value (n), NULL));
+			continue;
+		}
+	}
+
+	/* parse applications */
+	for (app_n = root->children->next; app_n != NULL; app_n = app_n->next) {
+		_cleanup_object_unref_ AsApp *app = NULL;
+		if (app_n->children == NULL)
+			continue;
+		app = as_app_new ();
+		if (!as_app_node_parse_dep11 (app, app_n, error))
+			return FALSE;
+		if (as_app_get_id (app) != NULL)
+			as_store_add_app (store, app);
+	}
+	return TRUE;
+}
+
 /**
  * as_store_from_file:
  * @store: a #AsStore instance.
@@ -550,7 +706,8 @@ as_store_from_root (AsStore *store,
  * @cancellable: a #GCancellable.
  * @error: A #GError or %NULL.
  *
- * Parses an AppStream XML file and adds any valid applications to the store.
+ * Parses an AppStream XML or DEP-11 YAML file and adds any valid applications
+ * to the store.
  *
  * If the root node does not have a 'origin' attribute, then the method
  * as_store_set_origin() should be called *before* this function if cached
@@ -567,11 +724,18 @@ as_store_from_file (AsStore *store,
 		    GCancellable *cancellable,
 		    GError **error)
 {
+	_cleanup_free_ gchar *filename = NULL;
 	_cleanup_error_free_ GError *error_local = NULL;
 	_cleanup_node_unref_ GNode *root = NULL;
 
 	g_return_val_if_fail (AS_IS_STORE (store), FALSE);
 
+	/* a DEP-11 file */
+	filename = g_file_get_path (file);
+	if (g_strstr_len (filename, -1, ".yml") != NULL)
+		return as_store_load_yaml_file (store, file, cancellable, error);
+
+	/* an AppStream XML file */
 	root = as_node_from_file (file,
 				  AS_NODE_FROM_XML_FLAG_LITERAL_TEXT,
 				  cancellable,
@@ -1054,6 +1218,7 @@ as_store_monitor_directory (AsStore *store,
 static gboolean
 as_store_load_app_info (AsStore *store,
 			const gchar *path,
+			const gchar *format,
 			GCancellable *cancellable,
 			GError **error)
 {
@@ -1061,31 +1226,31 @@ as_store_load_app_info (AsStore *store,
 	_cleanup_dir_close_ GDir *dir = NULL;
 	_cleanup_error_free_ GError *error_local = NULL;
 	_cleanup_free_ gchar *icon_root = NULL;
-	_cleanup_free_ gchar *path_xml = NULL;
+	_cleanup_free_ gchar *path_md = NULL;
 
 	/* watch the directory for changes */
 	if (!as_store_monitor_directory (store, path, cancellable, error))
 		return FALSE;
 
 	/* search all files */
-	path_xml = g_build_filename (path, "xmls", NULL);
-	if (!g_file_test (path_xml, G_FILE_TEST_EXISTS))
+	path_md = g_build_filename (path, format, NULL);
+	if (!g_file_test (path_md, G_FILE_TEST_EXISTS))
 		return TRUE;
-	dir = g_dir_open (path_xml, 0, &error_local);
+	dir = g_dir_open (path_md, 0, &error_local);
 	if (dir == NULL) {
 		g_set_error (error,
 			     AS_STORE_ERROR,
 			     AS_STORE_ERROR_FAILED,
 			     "Failed to open %s: %s",
-			     path_xml, error_local->message);
+			     path_md, error_local->message);
 		return FALSE;
 	}
 	icon_root = g_build_filename (path, "icons", NULL);
 	while ((tmp = g_dir_read_name (dir)) != NULL) {
-		_cleanup_free_ gchar *filename_xml;
-		filename_xml = g_build_filename (path_xml, tmp, NULL);
+		_cleanup_free_ gchar *filename_md;
+		filename_md = g_build_filename (path_md, tmp, NULL);
 		if (!as_store_load_app_info_file (store,
-						  filename_xml,
+						  filename_md,
 						  icon_root,
 						  cancellable,
 						  error))
@@ -1269,7 +1434,7 @@ as_store_load_installed (AsStore *store,
 			return FALSE;
 		}
 
-		/* do not load applications with NoDisplay=true */
+		/* do not load applications with vetos */
 		if ((flags & AS_STORE_LOAD_FLAG_ALLOW_VETO) == 0 &&
 		    as_app_get_vetos(app)->len > 0)
 			continue;
@@ -1375,7 +1540,9 @@ as_store_load (AsStore *store,
 		dest = g_build_filename (priv->destdir ? priv->destdir : "/", tmp, NULL);
 		if (!g_file_test (dest, G_FILE_TEST_EXISTS))
 			continue;
-		if (!as_store_load_app_info (store, dest, cancellable, error))
+		if (!as_store_load_app_info (store, dest, "xmls", cancellable, error))
+			return FALSE;
+		if (!as_store_load_app_info (store, dest, "yaml", cancellable, error))
 			return FALSE;
 	}
 
